# File: test_isAnDO.py
# Project: tests
# File Created: Tuesday, 30th June 2020 10:50:05 am
# Author: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Last Modified: Thursday, 2nd July 2020 1:33:43 pm
# Modified By: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Copyright - 2020 MIT, Institue de neurosciences de la Timone


import unittest
import ando.engine as andoE
import os
currpath = os.path.dirname(os.path.abspath(__file__))


class test_AnDO(unittest.TestCase):

    def setUp(self):
        pass

    '''testing ds001 folder'''
    def test_AnDO_dataset_1(self):
        directory = currpath+"/ds001/Data/Landing"
        self.assertEqual(andoE.is_AnDO(directory), False)

    '''testing ds002 folder'''
    def test_AnDO_dataset_2(self):
        directory = currpath+"/ds002/data/exp-Landing"
        self.assertEqual(andoE.is_AnDO(directory), False)

    '''testing ds003 folder'''
    def test_AnDO_dataset_3(self):
        directory = currpath+"/ds003/data/my_experiment"
        self.assertEqual(andoE.is_AnDO(directory), False)

    '''testing ds004 folder'''
    def test_AnDO_dataset_4(self):
        directory = currpath+"/ds004/data/newexp_vision"
        self.assertEqual(andoE.is_AnDO(directory), False)

    '''testing ds005 folder'''
    def test_AnDO_dataset_5(self):
        directory = currpath+"/ds005/data/my_experiment"
        self.assertEqual(andoE.is_AnDO(directory), False)

    '''testing ds006 folder'''
    def test_AnDO_dataset_6(self):
        directory = currpath+"/ds006/data/Landing"
        self.assertEqual(andoE.is_AnDO(directory), False)

    '''testing ds007 folder'''
    def test_AnDO_dataset_7(self):
        directory = currpath+"/ds007/data/exp-Landing/"
        self.assertEqual(andoE.is_AnDO(directory), False)


if __name__ == '__main__':
    unittest.main()



# File: engine.py
# Project: ando
# File Created: Tuesday, 30th June 2020 10:50:05 am
# Author: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Last Modified: Thursday, 2nd July 2020 1:32:33 pm
# Modified By: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Copyright - 2020 MIT, Institue de neurosciences de la Timone

# !/usr/bin/python
# -*- coding: utf-8 -*-

import os
import json
import re
import pathlib
from ando.error import ExperimentError, SourceError, SourceNotFound, \
    SessionError, SubjectError, MetaDataError, DerivativeDataError, \
    RawDataError


dir_rules = os.path.join(os.path.dirname(__file__)) + '/rules/'


def parse_all_path(nested_list_of_dir):
    """
    Transform this
    [
        ['Landing', 'sub-anye', '180116_001_m_anye_land-001', 'source'],
        ['Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source'],
        ['Landing', 'sub-enyo'],
        ['Landing', 'sub-enyo', '180116_001_m_enyo_land-001']
    ]
    to
    [
        ['Landing', 'sub-anye', '180116_001_m_anye_land-001', 'source'],
        ['Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source'],
    ]
    Checking for the longest chain with the same sub chain
    """

    def _test_is_included(my_list_of_lists, list_elem):
        for my_list_elem in my_list_of_lists:
            if all([val[0] == val[1] for val in zip(my_list_elem,
                   list_elem)]):
                return True

        return False

    def _merge_duplicates(my_list_of_lists, max_length=3):
        """Transform this
        [
            ['Landing', 'sub-anye', '180116_001_m_anye_land-001', 'source']
            ['Landing', 'sub-anye', '180116_001_m_anye_land-001', 'metadata']
            ['Landing', 'sub-anye', '180116_001_m_anye_land-001', 'rawdata']
            ['Landing', 'sub-anye', '180116_001_m_anye_land-001','derivatives']
        ]
        to
        [
            [
                'Landing', 'sub-anye',
                '180116_001_m_anye_land-001',
                'rawdata',
                'metadata',
                'derivatives',
                'source'
            ]
        ]

        Args:
            my_list_of_lists ([list]): [list of path to process]
            max_length (int, optional): [number of folder in session directory
            coresponding torawdata metadata derivatives and sources].
            Defaults to 3.

        Returns:
            [list]: [list of concatenate sub folder at the end of the list ]

        todo:
            This might have to be re-implemented more efficiently.
        """
        merged_list = []
        for my_list_elem in my_list_of_lists:
            simil_list = []
            for my_list_elem2 in my_list_of_lists:
                if all([val[0] == val[1] for i, val in
                        enumerate(zip(my_list_elem, my_list_elem2))
                        if i < max_length]):

                    simil_list.append(my_list_elem2)

            if len(simil_list) > 1:
                new_list = simil_list[0][:max_length]
                for remain_list in simil_list:
                    new_list.append(remain_list[max_length])
                merged_list.append(new_list)

            else:
                merged_list.append(simil_list[0])

        return merged_list

    new_list_of_lists = []
    for list_elem in sorted(nested_list_of_dir,
                            key=lambda sublist: len(sublist), reverse=True):
        if not _test_is_included(new_list_of_lists, list_elem):
            new_list_of_lists.append(list_elem)
    # Removing duplicate
    new_list_of_lists = _merge_duplicates(new_list_of_lists)
    unique_data = [list(x) for x in set(tuple(x)for x in new_list_of_lists)]
    return unique_data


def create_nested_list_of_path(directory):
    """
    Function that get the path given in arg
    to create a list of path as follow
    take the last element of the path and walks through to get every sub
    dir as follow:
    /home/garciaj/AnDOChecker/checker/tests/ds001/Data/Landing/
    to
    [['Landing', 'sub-enya', 'y180116-land-001', 'Sources']]

    """

    list_of_dir = []
    # take the last folder pass in arg: tests/ds007/data/Landing -> Landing

    path = pathlib.PurePath(directory)
    sub = directory.split(path.name)[0]

    # take everything before last tests/ds007/data/Landing -> tests/ds007/data

    for (root, dirs, _) in os.walk(directory):
        for d in dirs:
            list_of_dir.append(os.path.join(root, d).replace(sub, ''))
    nested_list_of_dir = []

    for each in list_of_dir:
        nested_list_of_dir.append(each.split(os.sep))
    nested_list_of_dir_parsed = parse_all_path(nested_list_of_dir)
    return nested_list_of_dir_parsed


def is_AnDO_R(subpath, level, validate):
    """

    Check if file path adheres to AnDO.
    Main method of the validator. uses other class methods for checking
    different aspects of the directory path.


    Args:
        subpath ([list]): [list of names]
        level ([int]): [level of the folder , 0 : experiment, 1 : subject ...]
        validate ([list]): [list of boolean corresponding of each level]

    Returns:
        [list]: [validate : list of boolean corresponding of each level]
    """

    if level < len(subpath):
        if level == 0:
            validate.append(is_experiment(subpath[level]))
            is_AnDO_R(subpath, level + 1, validate)
        if level == 1:
            validate.append(is_subject(subpath[level]))
            is_AnDO_R(subpath, level + 1, validate)
        if level == 2:
            validate.append(is_session(subpath[level]))
            is_AnDO_R(subpath, level + 1, validate)
        if level == 3:
            validate.append(is_source(subpath[level]))
            is_AnDO_R(subpath, level + 1, validate)
        if level == 4:
            validate.append(is_metadata(subpath[level]))
            is_AnDO_R(subpath, level + 1, validate)
        if level == 5:
            validate.append(is_derivatives(subpath[level]))
            is_AnDO_R(subpath, level + 1, validate)
        if level == 6:
            validate.append(is_rawdata(subpath[level]))
            is_AnDO_R(subpath, level + 1, validate)
    elif level < 7:
        validate.append(False)
    return validate


def is_AnDO(directory):
    """

    Check if file path adheres to AnDO.
    Main method of the validator. uses other class methods for checking
    different aspects of the directory path.


    Args:
        directory ([str]): [names of the directory to check]

    Returns:
        [bool]: [does the directory adheres to the ando specification]
    """

    validate = []
    names = create_nested_list_of_path(directory)

    for item in names:
        is_AnDO_R(item, 0, validate)

    return all(validate)


def is_AnDO_verbose(directory):
    """
    Call the function is_AnDO_verbose_Format on every path in the list


    Args:
        directory ([str]): [names of the directory to check]

    Returns:
        [bool]: [does the directory adheres to the ando specification]
    """

    validate = []
    names = create_nested_list_of_path(directory)

    for item in names:
        validate.append(is_AnDO_verbose_Format(item))

    return any(validate)


def is_AnDO_verbose_Format(names):
    """
    Check if file path adheres to AnDO.
    Main method of the validator. uses other class methods for checking
    different aspects of the directory path.

    Args:
        names ([list]): [names to check]

    Raises:
        ExperimentError: raised if it doesn't the experiment rules
        SessionError: raised if it doesn't the session rules
        SubjectError: raised if it doesn't the subject rules
        SourceError: raised if it doesn't the source rules
        RawDataError: raised if it doesn't the rawdata rules
        DerivativeDataError: raised if it doesn't the derivatives rules
        MetaDataError: raised if it doesn't the metadata rules
        SourceNotFound: raised if it doesn't the source rules

    Returns:
        [bool]: true if error is found else false
        [out]: feedback for the web api
    """

    bool_error = 0
    out = list()
    # only error that exit without checking other folder

    if is_experiment(names[0]):
        bool_error = 0
    else:
        try:
            raise ExperimentError(names)
        except ExperimentError as e:
            print(e.strerror)
            out.append(e.strout)
            bool_error = 1
            return bool_error, out

    if is_session(names):
        bool_error = 0
    else:
        try:
            raise SessionError(names)
        except SessionError as e:
            print(e.strerror)
            out.append(e.strout)
            bool_error = 1
    if is_subject(names):
        bool_error = 0
    else:
        try:
            raise SubjectError(names)
        except SubjectError as e:
            print(e.strerror)
            out.append(e.strout)
            bool_error = 1
    if len(names) == 7:

        if is_source(names):
            bool_error = 0
        else:
            try:
                raise SourceError(names)
            except SourceError as e:
                print(e.strerror)
                out.append(e.strout)
                bool_error = 1
        if is_rawdata(names):
            bool_error = 0
        else:
            try:
                raise RawDataError(names)
            except RawDataError as e:
                print(e.strerror)
                out.append(e.strout)
                bool_error = 1
        if is_derivatives(names):
            bool_error = 0
        else:
            try:
                raise DerivativeDataError(names)
            except DerivativeDataError as e:
                print(e.strerror)
                out.append(e.strout)
                bool_error = 1
        if is_metadata(names):
            bool_error = 0
        else:
            try:
                raise MetaDataError(names)
            except MetaDataError as e:
                print(e.strerror)
                out.append(e.strout)
                bool_error = 1

    else:

        if not is_metadata(names):
            try:
                raise MetaDataError(names)
            except MetaDataError as e:
                print(e.strerror)
                out.append(e.strout)
                bool_error = 1
        if not is_rawdata(names):
            try:
                raise RawDataError(names)
            except RawDataError as e:
                print(e.strerror)
                out.append(e.strout)
                bool_error = 1
        if not is_derivatives(names):
            try:
                raise DerivativeDataError(names)
            except DerivativeDataError as e:
                print(e.strerror)
                out.append(e.strout)
                bool_error = 1
        if not is_source(names):
            try:
                raise SourceNotFound(names)
            except SourceNotFound as e:
                print(e.strerror)
                out.append(e.strout)
                bool_error = 1

    return bool_error, out


def is_experiment(names):
    """[Check names follows experiment rules]

    Args:
        names ([str]): [names founds in the path]

    Returns:
        [type]: [true or false ]
    """

    regexps = get_regular_expressions(dir_rules
                                      + 'experiment_rules.json')
    conditions = []
    if type(names) == str:

        conditions.append([re.compile(x).search(names) is not None
                          for x in regexps])
    elif type(names) == list:

        for word in names:
            conditions.append([re.compile(x).search(word) is not None
                              for x in regexps])

    return any(flatten(conditions))


def is_rawdata(names):
    """[Check names follows rawdata rules]

    Args:
        names ([str]): [names founds in the path]

    Returns:
        [bool]: [true or false ]
    """

    regexps = get_regular_expressions(dir_rules
                                      + 'rawdata_rules.json')
    conditions = []

    if type(names) == str:

        conditions.append([re.compile(x).search(names) is not None
                          for x in regexps])
    elif type(names) == list:

        for word in names:
            conditions.append([re.compile(x).search(word) is not None
                              for x in regexps])

        # print(flatten(conditions))

    return any(flatten(conditions))


def is_metadata(names):
    """[Check names follows metadata rules]

    Args:
        names ([str]): [names founds in the path]

    Returns:
        [bool]: [true or false ]
    """

    regexps = get_regular_expressions(dir_rules
                                      + 'metadata_rules.json')
    conditions = []

    if type(names) == str:

        conditions.append([re.compile(x).search(names) is not None
                          for x in regexps])
    elif type(names) == list:

        for word in names:
            conditions.append([re.compile(x).search(word) is not None
                              for x in regexps])

        # print(flatten(conditions))

    return any(flatten(conditions))


def is_derivatives(names):
    """[Check names follows derivatives rules]

    Args:
        names ([str]): [names founds in the path]

    Returns:
        [bool]: [true or false ]
    """

    regexps = get_regular_expressions(dir_rules
                                      + 'derivatives_rules.json')
    conditions = []

    if type(names) == str:

        conditions.append([re.compile(x).search(names) is not None
                          for x in regexps])
    elif type(names) == list:

        for word in names:
            conditions.append([re.compile(x).search(word) is not None
                              for x in regexps])

        # print(flatten(conditions))

    return any(flatten(conditions))


def is_session(names):
    """[Check names follows session rules]

    Args:
        names ([str]): [names founds in the path]

    Returns:
        [bool]: [true or false ]
    """

    regexps = get_regular_expressions(dir_rules + 'session_rules.json')
    conditions = []
    if type(names) == str:
        conditions.append([re.compile(x).search(names) is not None
                          for x in regexps])
    elif type(names) == list:

        for word in names:
            conditions.append([re.compile(x).search(word) is not None
                              for x in regexps])

        # print(flatten(conditions))

    return any(flatten(conditions))


def is_subject(names):
    """[Check names follows subject rules]

    Args:
        names ([str]): [names founds in the path]

    Returns:
        [bool]: [true or false ]
    """

    regexps = get_regular_expressions(dir_rules + 'subject_rules.json')
    conditions = []
    if type(names) == str:
        conditions.append([re.compile(x).search(names) is not None
                          for x in regexps])
    elif type(names) == list:

        for word in names:
            conditions.append([re.compile(x).search(word) is not None
                              for x in regexps])

        #  print(flatten(conditions))

    return any(flatten(conditions))


def is_source(names):
    """[Check names follows sources rules]

    Args:
        names ([str]): [names founds in the path]

    Returns:
        [bool]: [true or false ]
    """

    regexps = get_regular_expressions(dir_rules + 'source_rules.json')
    conditions = []
    if type(names) == str:
        conditions.append([re.compile(x).search(names) is not None
                          for x in regexps])
    elif type(names) == list:

        for word in names:
            conditions.append([re.compile(x).search(word) is not None
                              for x in regexps])

        #  print(flatten(conditions))

    return any(flatten(conditions))


def get_regular_expressions(fileName):
    '''
    https://github.com/bids-standard/bids-validator/tree/master/bids-validator/

    using function to read regex in rule files

    '''

    regexps = []

    with open(fileName, 'r') as f:
        rules = json.load(f)

    for key in list(rules.keys()):
        rule = rules[key]

        regexp = rule['regexp']

        if 'tokens' in rule:
            tokens = rule['tokens']

            for token in list(tokens):
                regexp = regexp.replace(token, '|'.join(tokens[token]))

        regexps.append(regexp)

    return regexps


def flatten(seq):
    """
    Format list the proper way
    example:
    [[x],[y],[z]]--->[x,y,z]
    :param seq: list to format
    """

    list_flaten = []
    for elt in seq:
        t = type(elt)
        if t is tuple or t is list:
            for elt2 in flatten(elt):
                list_flaten.append(elt2)
        else:
            list_flaten.append(elt)
    return list_flaten



# File: test_Rules.py
# Project: tests
# File Created: Tuesday, 30th June 2020 10:50:05 am
# Author: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Last Modified: Thursday, 2nd July 2020 1:35:37 pm
# Modified By: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Copyright - 2020 MIT, Institue de neurosciences de la Timone


import unittest
import ando.engine as andoE


class Test_Rules(unittest.TestCase):

    def setUp(self):
        pass

    '''testing session rules'''
    def test_session_rules(self):
        list = ["ses-18000116_001_land-001"]
        self.assertEqual(andoE.is_session(list), True)

    def test_session_rules_1(self):
        list = ["180116_m_enya_land-001"]
        self.assertEqual(andoE.is_session(list), False)

    def test_session_rules_2(self):
        list = ["180116_001_m_land-001"]
        self.assertEqual(andoE.is_session(list), False)

    def test_session_rules_3(self):
        list = ["ses-180116_001_m_land_001"]
        self.assertEqual(andoE.is_session(list), False)

    '''testing source rules'''
    def test_source_rules(self):
        list = ["source"]
        self.assertEqual(andoE.is_source(list), True)

    def test_source_rules_1(self):
        list = ["Source"]
        self.assertEqual(andoE.is_source(list), False)

    def test_source_rules_2(self):
        list = ["sources"]
        self.assertEqual(andoE.is_source(list), False)

    '''testing subject rules'''
    def test_subject_rules(self):
        list = ["sub-001"]
        self.assertEqual(andoE.is_subject(list), True)

    def test_subject_rules_1(self):
        list = ["sub_001"]
        self.assertEqual(andoE.is_subject(list), False)

    def test_subject_rules_2(self):
        list = ["Sub_001"]
        self.assertEqual(andoE.is_subject(list), False)


if __name__ == '__main__':
    unittest.main()



# File: server.py
# Project: Ando_web
# File Created: Wednesday, 1st July 2020 4:10:50 pm
# Author: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Last Modified: Thursday, 2nd July 2020 1:35:59 pm
# Modified By: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Copyright - 2020 MIT, Institue de neurosciences de la Timone


from flask import Flask, jsonify, render_template, request
import os
import ando.engine as a
app = Flask(__name__)


@app.route('/')
def index():
    return render_template('index.html')


@app.route('/_AnDOchecker', methods=["GET", "POST"])
def _AnDOchecker():
    """
    Take all the file selected and send it to the API via json : :
    [
    data : [
            'exp-Landing/sub-anye/180116_001_m_anye_land-001/Touch.txt',
            'exp-Landing/sub-enya/180116_001_m_enya_land-001/source/Touch.txt',
            'exp-Landing/sub-enya/180116_001_m_enya_land-001/Touch.txt',
            'exp-Landing/sub-enyo/180116_001_m_enyo_land-001/Touch.txt'
            ]
     ]

    Returns:
        [json]: return a json response with the feedback coresponding
         to the verbose mod of the AnDO Checker
        [
            "out" : [list] all error repported by the ando Checker function,
            "result": [bool] true: if an error occurred , false : if nothing
             is repported
        ]

    TODO: why not try to use jinja2 template .
    """

    if request.method == 'POST':
        data = request.json
        list_of_folders_names = list()
        list_of_path = list()
        for file in data:
            """
            Deleting files from path
            """
            list_of_path.append(os.path.dirname(file))

        for file in list_of_path:
            """
            Transform path to list of folders names :
            'exp-Landing/sub-enyo/180116_001_m_enyo_land-001/"
            to
            ['exp-Landing','sub-enyo','180116_001_m_enyo_land-001']
            """
            list_of_folders_names.append(file.split(os.sep))

        list_of_folders_names = a.parse_all_path(list_of_folders_names)
        for item in list_of_folders_names:
            result, feedback = (a.is_AnDO_verbose_Format(item))
        # print(out)
        return jsonify({
            "feedback": feedback,
            "result": result
         })


if __name__ == "__main__":
    app.run()



# File: test_engine.py
# Project: tests
# File Created: Tuesday, 30th June 2020 10:50:05 am
# Author: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Last Modified: Thursday, 2nd July 2020 1:33:13 pm
# Modified By: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Copyright - 2020 MIT, Institue de neurosciences de la Timone


import unittest
import ando.engine as andoE
import os
path = os.getcwd()


class test_AnDO(unittest.TestCase):
    """Overall check of the rules given in the rules directory

    Args:
        unittest ([unittest]): [check every level of a given list of path ]
    """

    def setUp(self):
        pass

    def test_AnDO_Func_experiment_level(self):
        """
        Check if the experiment level follow the rules given in
        rules/experiment_rules.json
        """
        names = list()
        names = ['Landing', 'sub-anye', '180116_001_m_anye_land-001',
                 'source']
        validate = list()
        self.assertEqual(all(andoE.is_AnDO_R(names, 0, validate)), False)

    def test_AnDO_Func_subject_level(self):
        """
        Check if the subject level follow the rules given in
        rules/subject_rules.json
        """
        names = list()
        names = ['exp-Landing', 'anye', 'sess-180116_001_m_anye_land-001',
                 'source']
        validate = list()
        self.assertEqual(all(andoE.is_AnDO_R(names, 0, validate)), False)

    def test_AnDO_Func_session_level(self):
        """
        Check if the session level follow the rules given in
        rules/session_rules.json
        """
        names = list()
        names = ['exp-Landing', 'sub-anye', '180116_001_m_anye_land-001',
                 'source']
        validate = list()
        self.assertEqual(all(andoE.is_AnDO_R(names, 0, validate)), False)

    def test_AnDO_Func_sources_level(self):
        """
        Check if the sources level follow the rules given in
        rules/sources_rules.json
        """
        names = list()
        names = ['exp-Landing', 'sub-anye', 'sess-180116_001_m_anye_land-001',
                 'sources']
        validate = list()
        self.assertEqual(all(andoE.is_AnDO_R(names, 0, validate)), False)


if __name__ == '__main__':
    unittest.main()



# File: checker.py
# Project: ando
# File Created: Tuesday, 30th June 2020 10:50:05 am
# Author: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Last Modified: Thursday, 2nd July 2020 1:29:58 pm
# Modified By: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Copyright - 2020 MIT, Institue de neurosciences de la Timone


# !/usr/bin/python
# -*- coding: utf-8 -*-


import os
import argparse

from ando.engine import is_AnDO, is_AnDO_verbose

dir_rules = os.path.join(os.path.dirname(__file__)) + 'rules/'

if __name__ == '__main__':
    """
    Main function :

    usage: AnDOChecker.py [-h] [-v] pathToFolder

        positional arguments:
        path           Path to your folder

        optional arguments:
        -h, --help     show this help message and exit
        -v, --verbose  increase output verbosity

    """
    # add argparse for verbose option

    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='increase output verbosity')
    parser.add_argument('path', help='Path to your folder ')
    args = parser.parse_args()

    if args.verbose:
        try:
            directory = args.path
        except IndexError:
            directory = '.'
        error = is_AnDO_verbose(directory)

        if error == 1:
            print("\n" +
                  directory +
                  ": Is Not validated by AnDOChecker")
        else:
            print("\n" +
                  directory +
                  ": Is validated by AnDOChecker")
    else:
        try:
            directory = args.path
        except IndexError:

            directory = '.'

        error_not_found = is_AnDO(directory)
        if not error_not_found:
            print("\n" +
                  directory +
                  ": Is Not validated by AnDOChecker")
        else:
            print("\n" +
                  directory +
                  ": Is validated by AnDOChecker")



# File: test_parse_all_path.py
# Project: tests
# File Created: Tuesday, 30th June 2020 10:50:05 am
# Author: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Last Modified: Thursday, 2nd July 2020 1:33:58 pm
# Modified By: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Copyright - 2020 MIT, Institue de neurosciences de la Timone


import unittest
import ando.engine as andoE
import os
path = os.getcwd()


class test_parse_all_path(unittest.TestCase):
    """Test the function parse_all_path

    Args:
        unittest (unittest): [parse_all_path should  Transform this
    [
        ['Landing', 'sub-anye', '180116_001_m_anye_land-001', 'source'],
        ['Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source'],
        ['Landing', 'sub-enyo'],
        ['Landing', 'sub-enyo', '180116_001_m_enyo_land-001']
    ]
    to
    [
        ['Landing', 'sub-anye', '180116_001_m_anye_land-001', 'source'],
        ['Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source'],
    ]
    Checking for the longest chain with the same sub chain]
    """
    def setUp(self):
        pass
    
    def test_parse_all_path(self):# noqa: E501
        result=[['exp-Landing', 'sub-enyo', '180116_001_m_enyo_land-001'],['exp-Landing', 'sub-anye', '180116_001_m_anye_land-001'],['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source']]
        list1=[['exp-Landing', 'sub-anye', '180116_001_m_anye_land-001'], ['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source'], ['exp-Landing', 'sub-enyo'], ['exp-Landing', 'sub-enyo', '180116_001_m_enyo_land-001']]
        e=andoE.parse_all_path(list1)

        self.assertEqual(e.sort(),result.sort())
    # flake8: noqa: E501
    def test_parse_all_path_1(self):
        result=[['exp-Landing', 'sub-anye', '180116_001_m_anye_land-001'], ['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source'], ['exp-Landing', 'sub-enyo', '180116_001_m_enyo_land-001'], ['exp-Landing', 'sub-anye', '180116_001_m_anye_land-001'], ['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001'], ['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source'], ['exp-Landing', 'sub-enyo', '180116_001_m_enyo_land-001']]
        list1=[['exp-Landing', 'sub-anye', '180116_001_m_anye_land-001'], ['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source'], ['exp-Landing', 'sub-enyo'], ['exp-Landing', 'sub-enyo', '180116_001_m_enyo_land-001']]
        e=andoE.parse_all_path(list1)

        self.assertEqual(e.sort(),result.sort())
    # flake8: noqa: E501
    def test_parse_all_path_2(self):
        list1=[['exp-Landing', 'sub-anye', '180116_001_m_anye_land-001'],
            ['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source'],
            ['exp-Landing', 'sub-enyo'],
            ['exp-Landing', 'sub-enyo', '180116_001_m_enyo_land-001'],
            ['exp-Landing', 'sub-enyo', '180116_001_m_enyo_land-002'],
            ['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001', 'derivatives'],
            ['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001', 'metadata'],
            ['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001', 'rawdata'] ]

        result=[['exp-Landing', 'sub-enyo', '180116_001_m_enyo_land-001'],
                ['exp-Landing', 'sub-enyo', '180116_001_m_enyo_land-002'],
                ['exp-Landing', 'sub-anye', '180116_001_m_anye_land-001'],
                ['exp-Landing', 'sub-enya', '180116_001_m_enya_land-001', 'source', 'rawdata', 'derivatives', 'metadata']]

        e=andoE.parse_all_path(list1)
        self.assertEqual(e.sort(),result.sort())


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os

import re

verstr = "unknown"
try:
    verstrline = open('ando/_version.py', "rt").read()
except EnvironmentError:
    pass # Okay, there is no version file.
else:
    VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
    mo = re.search(VSRE, verstrline, re.M)
    if mo:
        verstr = mo.group(1)
    else:
        raise RuntimeError("unable to find version in yourpackage/_version.py")



setup(
    name="AnDOChecker",
    version=verstr,
    packages=find_packages(),
    author="Jeremy Garcia, Sylvain Takerkart",
    description="Checks the validity of a directory with respect to the ANimal Data Organization (ANDO) specifications ",
    license='MIT',
    install_requires=[]
)



# File: error.py
# Project: ando
# File Created: Tuesday, 30th June 2020 10:50:05 am
# Author: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Last Modified: Thursday, 2nd July 2020 1:31:40 pm
# Modified By: garcia.j (Jeremy.garcia@univ-amu.fr)
# -----
# Copyright - 2020 MIT, Institue de neurosciences de la Timone


# !/usr/bin/python
# -*- coding: utf-8 -*-

"""[Exception raise when the name doesn't follow the AnDO specification

    self.strerror : corresponding to the output in CLI mod
    self.strout : corresponding to the output of web mod by returning
     html

    ]
"""

# flake8: noqa: E501
class ExperimentError(Exception):
    """
    Exception raise when the name doesn't follow the AnDO specification


    Args:
        Exception ([Exception]): [raised in engine.py]
    """

    def __init__(self, arg):
        names = arg
        self.strerror = 'Level 1 error [experiment folder] at : ' + names[0] + '\n' \
            + '  It should follow the exp-NAME format, where:\n' \
            + '    - NAME is a string designating the name of your experiment\n'
        self.strout = '<div class="card"><div class="card-header bg-danger text-white">' \
                + '1 error found at Source folder level.  </div><div class="card-body"> ' \
                + '<h4 class="em-header clearfix"><strong class="em-header pull-left">Error 4 type ' \
                + "[Experiment folder error] at : "+names[0]+" </strong></h4><br><b><i>" \
                + '</b></i>It should follow the exp-NAME format, where:  '\
                + ' <ul><li> NAME is a string designating the name of your experiment</li><ul></div></div>'

class SubjectError(Exception):
    """Exception raise when the name doesn't follow the AnDO specification of subject level


    Args:
        Exception ([Exception]): [raised in engine.py]
    """

    def __init__(self, arg):
        names = arg
        self.strerror = 'Level 2 error [subject folder] at : ' + names[1] + '\n' \
            + '  It should follow the sub-ID format, where:\n' \
            + '    - ID is a string designating the IDentifier of the animal\n'
        self.strout = '<div class="card"><div class="card-header bg-danger text-white">' \
                + '1 error found at Source folder level.  </div><div class="card-body"> ' \
                + '<h4 class="em-header clearfix"><strong class="em-header pull-left">Error 4 type ' \
                + "[Subject folder error] at : "+names[0]+" </strong></h4><br><b><i>" \
                + '</b></i>It should follow the sub-ID format, where:  '\
                + ' <ul><li> ID is a string designating the IDentifier of the animal</li><ul></div></div>'

class SessionError(Exception):
    """Exception raise when the name doesn't follow the AnDO specification of session level


    Args:
        Exception ([Exception]): [raised in engine.py]
    """

    def __init__(self, arg):
        names = arg
        self.strerror = 'Level 3 error [session folder] at : ' + names[2] + '\n' \
            + '  It should follow the ses-YYYYMMDD_XXX_BBBB format, where:\n' \
            + '    - ‘ses-’ is an imposed prefix\n' \
            + '    - ‘YYYYMMDD’ is the date of the session (8 digits, for instance 20180430 for April 30, 2018)\n' \
            + '    - XXX is the number of the session acquired on that date (3 digits, for instance 001 for the first session)\n' \
            + '    - BBBB is a string freely usable by the research group / user\n' \
            + '      (is a string freely usable by the research group / user (for instance to add extra info on \n' \
            + '      the version of the experimental protocol, on the type of preparation, on the user-friendly name of the animal etc.);\n' \
            + '      this string cannot contain the underscore character.\n'
        self.strout = '<div class="card"><div class="card-header bg-danger text-white">' \
                + '1 error found at Source folder level.  </div><div class="card-body"> ' \
                + '<h4 class="em-header clearfix"><strong class="em-header pull-left">Error 4 type ' \
                + "[session folder error] at : "+names[2]+" </strong></h4><br><b><i>" \
                + '</b></i> A folder called  <i><b>"session" </i></b>'\
                + ' should be present in the session folder.</div></div>'

class SourceError(Exception):
    """Exception raise when the name doesn't follow the AnDO specification of error level


    Args:
        Exception ([Exception]): [raised in engine.py]
    """

    def __init__(self, arg):
        names = arg
        self.strerror = 'Level 4 error [source folder] at : ' + names[2] + '\n' \
            + '  A single folder called source is authorized within a session folder\n'

        self.strout = '<div class="card"><div class="card-header bg-danger text-white">' \
                + '1 error found at Source folder level.  </div><div class="card-body"> ' \
                + '<h4 class="em-header clearfix"><strong class="em-header pull-left">Error 4 type ' \
                + "[source folder error] at : "+names[2]+" </strong></h4><br><b><i>" \
                + '</b></i> A folder called  <i><b>"source" </i></b>'\
                + ' should be present in the session folder. OR does not follow specification</div></div>'
class RawDataError(Exception):
    """Exception raise when the name doesn't follow the AnDO specification of rawdata level


    Args:
        Exception ([Exception]): [raised in engine.py]
    """

    def __init__(self, arg):
        names = arg
        self.strerror = 'Level 4 error [rawdata folder missing]\n' \
            + '  A folder called rawdata should be present in the session folder ' + names[2] + '\n'
        self.strout = '<div class="card"><div class="card-header bg-danger text-white">' \
                + '1 error found at Source folder level.  </div><div class="card-body"> ' \
                + '<h4 class="em-header clearfix"><strong class="em-header pull-left">Error 4 type ' \
                + "[rawdata folder error] at : "+names[2]+" </strong></h4><br><b><i>" \
                + '</b></i> A folder called  <i><b>"rawdata" </i></b>'\
                + ' should be present in the session folder.</div></div>'
class MetaDataError(Exception):
    """Exception raise when the name doesn't follow the AnDO specification of rawdata level


    Args:
        Exception ([Exception]): [raised in engine.py]
    """

    def __init__(self, arg):
        names = arg
        self.strerror = 'Level 4 error [metadata folder missing]\n' \
            + '  A folder called metadata should be present in the session folder ' + names[2] + '\n'
        self.strout = '<div class="card"><div class="card-header bg-danger text-white">' \
                + '1 error found at Source folder level.  </div><div class="card-body"> ' \
                + '<h4 class="em-header clearfix"><strong class="em-header pull-left">Error 4 type ' \
                + "[metadata folder error] at : "+names[2]+" </strong></h4><br><b><i>" \
                + '</b></i> A folder called  <i><b>"metadata" </i></b>'\
                + ' should be present in the session folder.</div></div>'
class DerivativeDataError(Exception):
    """Exception raise when the name doesn't follow the AnDO specification of derivatives level


    Args:
        Exception ([Exception]): [raised in engine.py]
    """

    def __init__(self, arg):
        names = arg
        self.strerror ='Level 4 error [derivatives folder missing]\n' \
            + '  A folder called derivatives should be present in the session folder ' + names[2] + '\n'
        self.strout = '<div class="card"><div class="card-header bg-danger text-white">' \
                + '1 error found at Source folder level.  </div><div class="card-body"> ' \
                + '<h4 class="em-header clearfix"><strong class="em-header pull-left">Error 4 type ' \
                + "[derivatives folder error] at : "+names[2]+" </strong></h4><br><b><i>" \
                + '</b></i> A folder called  <i><b>"derivatives" </i></b>'\
                + ' should be present in the session folder.</div></div>'

class SourceNotFound(Exception):
    """Exception raise when the name doesn't follow the AnDO specification of source level


    Args:
        Exception ([Exception]): [raised in engine.py]
     """
    def __init__(self, arg):
        names = arg
        self.strerror = 'Level 4 error [source folder missing]\n' \
            + '  A folder called source should be present in the session folder ' + names[2] + '\n'

        self.strout = '<div class="card"><div class="card-header bg-danger text-white">' \
            + '1 error found at Source folder level.  </div><div class="card-body"> ' \
            + '<h4 class="em-header clearfix"><strong class="em-header pull-left">Error 4 type ' \
            + "[Source folder error] at : "+names[2]+" </strong></h4><br><b><i>" \
            + '</b></i> A folder called  <i><b>"source" </i></b>'\
            + ' should be present in the session folder.</div></div>'

